@@ -1,5 +1,8 @@
 # 0.01 = 2021-03-06 = Initial version
 # 0.02 = 2021-07-19 = Added functionality to enter sensitive data without easy retrieving
+# 0.03 = 2021-08-04 Impossible easely adopt for 3.6, only for 3.7, because of some modules (for example: immutables) 
+#                   are not precompiled for 3.6 at pypi.org. So installing of Visual C/C++ (or MinGW) is required.
+# 0.04 = 2023-02-25 = AES.MODE_CBC -> AES.MODE_GCM accoding sonarcloud.io: Use secure mode and padding scheme.
 import os
 import base64
 
@@ -94,7 +97,7 @@ class n0Vault(dict):
         elif isinstance(xpath, str):
             self._vault.update({xpath: new_value})
         else:
-            raise TypeError(f"Expected: xpath: typing.Union[dict, str], new_value: str\nReceived: {type(xpath)=}, {type(new_value)=}")
+            raise TypeError(f"Expected: xpath: typing.Union[dict, str], new_value: str\nReceived: xpath:{type(xpath)}, new_value:{type(new_value)}")
             
         return self._vault
     # **********************************************************************************************
@@ -197,13 +200,13 @@ class n0Vault(dict):
                     # ******************************************************************************
                     # ******************************************************************************
                     if self.is_bit_set(0, 0b11) == 0b00:
-                        cipher = AES.new(self.__key, AES.MODE_CBC, cipher_iv)
+                        cipher = AES.new(self.__key, AES.MODE_GCM, cipher_iv)
                     elif self.is_bit_set(0, 0b11) == 0b01:
                         if not self.__password:
                             raise Exception(f"Password for loading is required")
                         cipher = AES.new(
-                            temp:=PBKDF2(self.__password, self.__key[:16]).read(32),    # 256-bit key
-                            AES.MODE_CBC,
+                            PBKDF2(self.__password, self.__key[:16]).read(32),    # 256-bit key
+                            AES.MODE_GCM,
                             cipher_iv
                         )
                     else:
@@ -241,11 +244,14 @@ class n0Vault(dict):
         if self.is_bit_set(2, 0b1):
             raise Exception(f"Saving of such n0Vault storage is forbidden")
         # ******************************************************************************************
-        def write_buffer(buffer, name):
+        def write_buffer(buffer: typing.Union[str, int], name_of_buffer: str):
+            n0debug_calc(buffer, name_of_buffer)
             if isinstance(buffer, str):
                 buffer = buffer.encode("utf-8")             # str -> bytes
-            if isinstance(buffer, int):
+            elif isinstance(buffer, int):
                 buffer = buffer.to_bytes(4, 'little')       # int32 -> bytes
+            else:
+                raise(Exception(f"Expected type str or int for '{buffer}', but got {type(buffer)}"))
             out_file.write(buffer)
         # ******************************************************************************************
         with open(new_vault_file_name or self.vault_file_name, "wb") as out_file:
@@ -256,10 +262,10 @@ class n0Vault(dict):
                     self.__flags = self.set_bits(bytes_array = self.__flags, bits_value = 0b01, bits_len = 2, bits_offset = 0)
                     cipher = AES.new(
                         PBKDF2(self.__password, self.__key[:16]).read(32),          # Generate 256-bit key
-                        AES.MODE_CBC
+                        AES.MODE_GCM
                     )
                 else:
-                    cipher = AES.new(self.__key, AES.MODE_CBC)
+                    cipher = AES.new(self.__key, AES.MODE_GCM)
                 write_buffer(self.__sign,  "sign")
                 if forbid_next_saving:
                     self.__flags = self.set_bits(bytes_array = self.__flags, bits_value = 0b1, bits_len = 1, bits_offset = 2)
